/*---------------------------------------------------------------------------------

	>> INSTITUT INFORMACNICH TECHNOLOGII
	>> FAKULTA TECHNOLOGICKA
	>> UNIVERZITA TOMASE BATI VE ZLINE					       
 
	Soubor	:	CCircle.cpp
	Popis	:	Rasterizace kruznice
	Autor	:	Jan Stalmach, stalmachjan@volny.cz 
	Obor	:	Informacni technologie, 2003-2004
	  
	Implementovany nasledujci algoritmy:
	
	Bresenhamuv kruznicovy interpolator
	Rotace bodu kolem stredu kruznice
	Algoritmus kresleni kruznice pomoci parametrickeho tvaru (DDA)
	
----------------------------------------------------------------------------------*/


#include <math.h>
#include "CCircle.h"

/*---------------------------------------------------------------------
 Konstruktor tridy Circle
-----------------------------------------------------------------------*/
CCircle::CCircle(float r=1.0, float g=1.0, float b=1.0) 
{ 
	Casovac = new Stopky;
	Cara	= new CLine(1,0,0);
	red   = r;
	green = g;
	blue  = b;
}


/*---------------------------------------------------------------------
 Nastavi vykreslovaci barvu
-----------------------------------------------------------------------*/
void CCircle::Set_RGB(float r, float g, float b)
{
	red   = r;
	green = g;
	blue  = b;
}


/*---------------------------------------------------------------------
 Algoritmus vyuzivajici parametrickeho vyjadreni kruznice

 x = x0 + polomer*cos(alfa)
 y = y0 + polomer*sin(alfa)
-----------------------------------------------------------------------*/
double CCircle::Parametricky_tvar(int x, int y, int polomer)
{
	Barva(red,green,blue);

	Casovac->Unpause();

	float	angle = PI / (4 * polomer);
	float	xp, yp, alfa;
	
	for (int i = 0; i <= polomer; i++) 
	{
		alfa = i * angle;
		xp = polomer * cos(alfa);
		yp = polomer * sin(alfa);

		// Na zaklade symetrie vykresli casti kruznice
		Bod(x + xp, y + yp);
        Bod(x + xp, y - yp);
        Bod(x + yp, y + xp);
        Bod(x + yp, y - xp);
        Bod(x - xp, y + yp);
        Bod(x - xp, y - yp);
        Bod(x - yp, y + xp);
        Bod(x - yp, y - xp);
	}

	Casovac->Pause();
	return Casovac->GetTime();
}

/*---------------------------------------------------------------------
 Algoritmus vyuzivajici rotaci usecky kolem stredu kruznice
-----------------------------------------------------------------------*/
double CCircle::Rotace(int x, int y, int polomer)
{
	Cara->Set_RGB(red,green,blue);

	Casovac->Unpause();

	int		steps;
	float	x1, y1, x2, y2, alfa;

	if (polomer<=30)
	{
		steps = 3; 
		alfa  = 50;
	} else

	if (polomer<=200)
	{
		steps = 6; 
		alfa  = 25;
	} else

	if (polomer>200)
	{
		steps = 12; 
		alfa  = 12.5;
	}

	float c_alfa = cos(alfa);
	float s_alfa = sin(alfa);

	x1 = polomer;
	y1 = 0;
	
	for (int i = 0; i < steps; i++) 
	{
		x2 = x1*c_alfa-y1*s_alfa;
		y2 = x1*s_alfa+y1*c_alfa;
		
		// Na zaklade symetrie vykresli casti kruznice
		Cara->Bresenham(x+x1,y+y1,x+x2,y+y2);
		Cara->Bresenham(x+x1,y-y1,x+x2,y-y2);
		Cara->Bresenham(x-x1,y-y1,x-x2,y-y2);
		Cara->Bresenham(x-x1,y+y1,x-x2,y+y2);
		Cara->Bresenham(x+y1,y+x1,x+y2,y+x2);
		Cara->Bresenham(x+y1,y-x1,x+y2,y-x2);
		Cara->Bresenham(x-y1,y-x1,x-y2,y-x2);
		Cara->Bresenham(x-y1,y+x1,x-y2,y+x2);

		x1 = x2;
		y1 = y2;
	}

	Casovac->Pause();
	return Casovac->GetTime();
}


/*---------------------------------------------------------------------
 Bresenhamuv kruznicovy interpolator
-----------------------------------------------------------------------*/
double CCircle::Bresenham(int x0, int y0, int polomer)
{
	Barva(red,green,blue);
	
	Casovac->Unpause();
		
    int x = 0, y = polomer;
	int p = 3 - 2*polomer;

	// Prvni 4 body
    Bod(x0, y0 + polomer);
    Bod(x0 + polomer, y0); 
    Bod(x0, y0 - polomer);
    Bod(x0 - polomer, y0);
 	
	// Kresleni kruznice
	while (x < y) 
	{
		// Na zaklade symetrie vykresli casti kruznice
		Bod(x0 + x, y0 + y);
        Bod(x0 + x, y0 - y); 
        Bod(x0 + y, y0 + x);
        Bod(x0 + y, y0 - x);
        Bod(x0 - x, y0 + y);
        Bod(x0 - x, y0 - y);
        Bod(x0 - y, y0 + x);
        Bod(x0 - y, y0 - x);

		if (p < 0) p += 4*x + 6;
		   else 
			{  
				p += 4*(x - y)+10;
				y--;
			}
        x++;
	
 	}

	Casovac->Pause();
	return Casovac->GetTime();
}