/*-----------------------------------------------------------------------------
 
    >> INSTITUT INFORMACNICH TECHNOLOGII
	>> FAKULTA TECHNOLOGICKA
	>> UNIVERZITA TOMASE BATI VE ZLINE
					        
   	Soubor	:	CCurve.h
	Popis	:	Modelovani parametrickych krivek
	Autor	:	Jan Stalmach, stalmachjan@volny.cz 
	Obor	:	Informacni technologie, 2003-2004
  
    Implementovany nasledujci algoritmy:

	Bezierova krivka
	Bezierova racionalni krivka
	Coonsova BSpline krivka
	Catmul-Rom krivka
	Fergusonova krivka
	NURBS

	Bezierova kubika iterativne
	Bezierova kubika rekurzivne
	Coonsova kubika
	Catmul-Rom kubika
	Fergusonova kubika

	deCasteljau algoritmus

 ------------------------------------------------------------------------------*/

#ifndef _CCURVE_H	
#define _CCURVE_H		

#include "CRaster.h"
#include "stopky.h"
#include "CLine.h"


// maximalni pocet kroku pro aproximaci krivek lomenou carou
//#define MAX_KROKU 160


/*----------------------------------------------
	Trida CCurve
------------------------------------------------*/
class CCurve: public CRaster {

protected:

	Stopky *Casovac;

	float red,green,blue;
	
	float Faktorial(int n);
	float Ni(int n,int i);
	float Basis(int n,int i,float t);

	bod  Souradnice_Bezier(bod *p,int n,float t);
	bod  Souradnice_Bezierova_Kubika_Iterativne(bod p1,bod p2,bod p3,bod p4, float t);
	void Souradnice_Bezierova_Kubika_Rekursivne(bod *b, int level); 
	bod  Souradnice_Coonsova_Kubika(bod p1,bod p2,bod p3,bod p4, float t);
	bod  Souradnice_Catmul_Rom_Kubika(bod p1,bod p2,bod p3,bod p4, float t);
	bod  Souradnice_Fergusonova_Kubika(bod p1,bod v1,bod v2,bod p2, float t);
	bod  Souradnice_Racionalni_Bezier(bod *p,int n,float t, float *m);

	void   BSpline_uzly(float *u,int n,int t);
	void   BSpline_bod(float *u,int n,int t,double v,bod *control,bod *output);
	void   Souradnice_BSpline(bod *inp,int n,float *uzly,int t,bod *outp,int res);
	void   Souradnice_NURBS(bod *inp,int n,float *uzly,float *vaha,int t,bod *outp,int res);
	void   NURBS_bod(float *u,float *vaha,int n,int t,double v,bod *control,bod *output);
	void   NURBS_uzly(int *u,int n,int t);

	double BSpline_baze(int k,int t,float *u,double v);

public:

	CCurve(float r, float g, float b, int max_kroku);
	~CCurve() {}

	CLine   *Cara;

	double Bezier(bod *p,int stp);
	double Racionalni_Bezier(bod *p, int stp, float *m);
	double BSplineCoons(bod *inp,int stp);
	double Catmul_Rom(bod *inp,int stp);
	double Ferguson(bod *inp,int stp);
	double BSpline(bod *inp, int n, int t);
	double NURBS(bod *inp,float *uzly, float *vaha,int n, int U_size);

	double Bezierova_Kubika_Iterativne(bod A, bod B, bod C, bod D); 
	double Bezierova_Kubika_Rekursivne(bod A, bod B, bod C, bod D);
	double Coonsova_Kubika(bod A, bod B, bod C, bod D);
	double Catmul_Rom_Kubika(bod A, bod B, bod C, bod D);
	double Fergusonova_Kubika(bod A, bod B, bod vA, bod vB);

	void deCasteljau(bod *p, bod *left, bod *right);

	void Ridici_Polygon(bod *p, int stp, float r, float g, float b);		
	void Set_RGB(float r, float g, float b);

	float *uzel;

	int MAX_KROKU;
};

#endif
