/*---------------------------------------------------------------------------------

	>> INSTITUT INFORMACNICH TECHNOLOGII
	>> FAKULTA TECHNOLOGICKA
	>> UNIVERZITA TOMASE BATI VE ZLINE					       
 
	Soubor	:	CElipse.cpp
	Popis	:	Rasterizace elipsy
	Autor	:	Jan Stalmach, stalmachjan@volny.cz 
	Obor	:	Informacni technologie, 2003-2004
	
	Implementovany nasledujci algoritmy:

	Bresenhamuv algoritmus pro kreleni elipsy
	Algoritmus vyuzivajici parametrickeho vyjadreni eplipsy
	Elipsa v obecne poloze

----------------------------------------------------------------------------------*/


#include <math.h>
#include "CElipse.h"

/*---------------------------------------------------------------------
 Konstruktor tridy Elipse
-----------------------------------------------------------------------*/
CElipse::CElipse(float r=1.0, float g=1.0, float b=1.0) 
{ 
	Casovac = new Stopky;
	red   = r;
	green = g;
	blue  = b;
}

/*---------------------------------------------------------------------
 Nastavi vykreslovaci barvu
-----------------------------------------------------------------------*/
void CElipse::Set_RGB(float r, float g, float b)
{
	red   = r;
	green = g;
	blue  = b;
}

/*---------------------------------------------------------------------
 Bresenhamuv algoritmus pro kreleni elipsy
-----------------------------------------------------------------------*/
double CElipse::Bresenham(int x0, int y0, int a, int b)
{
	Barva(red,green,blue);

	Casovac->Unpause();

	int x, y;
	int AA, AA2, BB, BB2;
	int pre, dx, dy;

	// Zaciname v 1.kvadrantu vlevo nahore
	x = 0;
	y = b;
	AA = a * a;        // 2.mocnina a
	BB = b * b;        // 2.mocnina b
	AA2 = AA >> 1 ;    // Dvojnasobek 2.mocniny a
	BB2 = BB >> 1 ;    // Dvojnasobek 2.mocniny b
	
	// Inicializace predikce
	pre = (BB - AA * b + AA) >> 2;
	dx = 0;
	dy = AA2 * b;

	// 1. cast algoritmu
	while(dx < dy)
	{
		// Vykresleni 4 symetrickych bodu
		Bod(x0 + x, y0 + y);
		Bod(x0 - x, y0 + y);
		Bod(x0 + x, y0 - y);
		Bod(x0 - x, y0 - y);
  
		if(pre >= 0)
		{
			y--;
			dy  -= AA2;
			pre -= dy;
		}

		x++;
		dx += BB2;
		pre += BB + dx;
	}

	
	// 2.cast algoritmu
	// Inicializace predikce
	pre += (((3 * (AA - BB)) >> 1) - dx - dy) >> 1;
	while(y >= 0)
	{
		// Vykresleni 4 symetrickych bodu
		Bod(x0 + x, y0 + y);
		Bod(x0 - x, y0 + y);
		Bod(x0 + x, y0 - y);
		Bod(x0 - x, y0 - y);

		if(pre <= 0)
		{
			x++;
		    dx  += BB2;
	    	pre += dx;
		}

		y--;
		dy -= AA2;
		pre += AA - dy;
	}

 	Casovac->Pause();
	return Casovac->GetTime();
}


/*---------------------------------------------------------------------
 Algoritmus vyuzivajici parametrickeho vyjadreni eplipsy

 x = x0 + a*cos(alfa)
 y = y0 + b*sin(alfa)
-----------------------------------------------------------------------*/
double CElipse::Parametricky_tvar(int x0, int y0, int a, int b)
{
	Barva(red,green,blue);

	Casovac->Unpause();

	int		steps;
	if (a>b) steps=a; else steps=b;

	float	angle = PI / (4*steps);
	float	xp, yp, alfa;

	steps*=2;
	
	for (int i = 0; i <= steps; i++) 
	{
		alfa = i * angle;
		xp = a * cos(alfa);
		yp = b * sin(alfa);

		// Symetricke vykresleni bodu
		Bod(x0 + xp, y0 + yp);
		Bod(x0 - xp, y0 + yp);
		Bod(x0 + xp, y0 - yp);
		Bod(x0 - xp, y0 - yp);
	}

	Casovac->Pause();
	return Casovac->GetTime();
}


/*---------------------------------------------------------------------
 Elipsa v obecne poloze

 Zalozeno na modifikaci algoritmu vyuzivajiciho parametrickeho 
 vyjadreni eplipsy a transformaci otacenim
-----------------------------------------------------------------------*/
double CElipse::Obecna_poloha(int x0, int y0, int a, int b, int uhel)
{
	Barva(red,green,blue);

	Casovac->Unpause();

	int		steps;
	if (a>b) steps=a; else steps=b;

	float	angle = PI / (4*steps);
	float	xp, yp, xpr, ypr, alfa;

	steps*=2;

	// Predvypocet cos a sin
	float   cos_a    = cos(RAD(uhel));
	float   sin_a    = sin(RAD(uhel));
	float   cos_a2   = cos(RAD(-uhel));
	float   sin_a2   = sin(RAD(-uhel));
	
	for (int i = 0; i <= steps; i++) 
	{
		alfa = i * angle;
		xp = a * cos(alfa);
		yp = b * sin(alfa);

		// Transformace otocenim o kladny uhel 
		xpr = xp*cos_a-yp*sin_a;
		ypr = xp*sin_a+yp*cos_a;

		// Symetricke body
        Bod(x0 + xpr, y0 + ypr);
		Bod(x0 - xpr, y0 - ypr);

		// Transformace otocenim o zaporny uhel 
		xpr = xp*cos_a2-yp*sin_a2;
		ypr = xp*sin_a2+yp*cos_a2;

		// Symetricke body
        Bod(x0 - xpr, y0 + ypr);
		Bod(x0 + xpr, y0 - ypr);

	}

	Casovac->Pause();
	return Casovac->GetTime();
}