/*---------------------------------------------------------------------------------

	>> INSTITUT INFORMACNICH TECHNOLOGII
	>> FAKULTA TECHNOLOGICKA
	>> UNIVERZITA TOMASE BATI VE ZLINE					       
 
	Soubor	:	CLine.cpp
	Popis	:	Rasterizace usecky
	Autor	:	Jan Stalmach, stalmachjan@volny.cz 
	Obor	:	Informacni technologie, 2003-2004
	  
	Implementovany nasledujci algoritmy:

	Digital Differencial Analyser (DDA)
	Bresenhamuv linearni interpolator
	Xiaolin Wu Line Two-Step

---------------------------------------------------------------------------------*/


#include <math.h>
#include "CLine.h"


/*---------------------------------------------------------------------
 Konstruktor tridy Line
-----------------------------------------------------------------------*/
CLine::CLine(float r=1.0, float g=1.0, float b=1.0) 
{ 
	Casovac = new Stopky;
	red   = r;
	green = g;
	blue  = b;
}


/*---------------------------------------------------------------------
 Nastavi vykreslovaci barvu
-----------------------------------------------------------------------*/
void CLine::Set_RGB(float r, float g, float b)
{
	red   = r;
	green = g;
	blue  = b;
}


/*---------------------------------------------------------------------
 DDA algoritmus
-----------------------------------------------------------------------*/
double CLine::DDA(int x0, int y0, int x1, int y1)
{   
	Barva(red,green,blue);
	
	Casovac->Unpause();

	int dy = y1 - y0;
	int dx = x1 - x0;
	float t = (float) 0.5;                      // Posunuti o konstantu t = 0.5 
	
	Bod(x0,y0);
	if (abs(dx) > abs(dy)) 
	{                                           // Sklon < 1
		float m = (float) dy / (float) dx;      // Vypocet smernice
		t += y0;
		dx = (dx < 0) ? -1 : 1;
		m *= dx;
		while (x0 != x1) 
		{
			x0 += dx;                           // Prictu dx
			t += m;                             // Prictu m
			Bod(x0, (int) t);
		}
	} else 
	{                                           // Sklon >= 1
		float m = (float) dx / (float) dy;      
		t += x0;
		dy = (dy < 0) ? -1 : 1;
		m *= dy;
		while (y0 != y1) 
		{
			y0 += dy;                          
			t += m;                            
			Bod( (int) t, y0);		
		}
	}

	Casovac->Pause();
	return Casovac->GetTime();
}


/*---------------------------------------------------------------------
 Bresenhamuv celociselny linearni interpolacni algoritmus
-----------------------------------------------------------------------*/
double CLine::Bresenham(int x0, int y0, int x1, int y1) 
{
	Barva(red,green,blue);
	
	Casovac->Unpause();

	int dy = y1 - y0;
	int dx = x1 - x0;
	int krokx, kroky;
	if (dy < 0) { dy = -dy;  kroky = -1; } else  kroky = 1; 
	if (dx < 0) { dx = -dx;  krokx = -1; } else  krokx = 1; 

	dy >>= 1;                                                  // dy je 2*dy
	dx >>= 1;												   // dx je 2*dx
	
	Bod(x0,y0);
	if (dx > dy) 
	{
		int predikace = dy - (dx >> 1);                        // dy - 2*dx
		while (x0 != x1) 
		{
			if (predikace >= 0) 
			{
				y0 += kroky;
				predikace -= dx;                               
			}
			x0 += krokx;
			predikace += dy;                                   
			Bod(x0,y0);
		}
	} else 
	{
		int predikace = dx - (dy >> 1);
		while (y0 != y1) 
		{
			if (predikace >= 0) 
			{
				x0 += krokx;
				predikace -= dy;
			}
			y0 += kroky;
			predikace += dx;
			Bod(x0,y0);
		}
	}

	Casovac->Pause();
	return Casovac->GetTime();
		
}


/*---------------------------------------------------------------------
 Xiaolin Wu Line 2-Step algoritmus
-----------------------------------------------------------------------*/
double CLine::TwoStep(int x0, int y0, int x1, int y1)
{
    Barva(red,green,blue);
	
	Casovac->Unpause();

	int dy = y1 - y0;
    int dx = x1 - x0;
    int stepx, stepy;

    if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; }
    if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }

    Bod( x0, y0);
    Bod( x1, y1);
    
	if (dx > dy)
	/**/	{
            int length = (dx - 1) >> 2;
            int extras = (dx - 1) & 3;
            int incr2 = (dy << 2) - (dx << 1);
            if (incr2 < 0) 
			{
                int c = dy << 1;
                int incr1 = c << 1;
                int d =  incr1 - dx;
                for (int i = 0; i < length; i++)
				{
                    x0 += stepx;
                    x1 -= stepx;
                    if (d < 0) 
					{					       	// Pattern:
                        Bod( x0, y0);			//
                        Bod( x0 += stepx, y0);	//  x o o
                        Bod( x1, y1);			//
                        Bod( x1 -= stepx, y1);
                        d += incr1;
                    } else 
					{
                        if (d < c) 
						{						            	// Pattern:
                            Bod( x0, y0);			        	//      o
                            Bod( x0 += stepx, y0 += stepy);		//  x o
                            Bod( x1, y1);			        	//
                            Bod( x1 -= stepx, y1 -= stepy);
                        } else 
						{
                            Bod( x0, y0 += stepy);			// Pattern:
                            Bod( x0 += stepx, y0);			//    o o 
                            Bod( x1, y1 -= stepy);			//  x
                            Bod( x1 -= stepx, y1);			//
                        }
                        d += incr2;
                    }
                }
                if (extras > 0) 
				{
                    if (d < 0)
					{
                        Bod( x0 += stepx, y0);
                        if (extras > 1) Bod( x0 += stepx, y0);
                        if (extras > 2) Bod( x1 -= stepx, y1);
                    } else
                    if (d < c)
					{
                        Bod( x0 += stepx, y0);
                        if (extras > 1) Bod( x0 += stepx, y0 += stepy);
                        if (extras > 2) Bod( x1 -= stepx, y1);
                    } else 
					{
                        Bod( x0 += stepx, y0 += stepy);
                        if (extras > 1) Bod( x0 += stepx, y0);
                        if (extras > 2) Bod( x1 -= stepx, y1 -= stepy);
                    }
                }
            } else 
			{
                int c = (dy - dx) << 1;
                int incr1 = c << 1;
                int d =  incr1 + dx;
                for (int i = 0; i < length; i++) 
				{
                    x0 += stepx;
                    x1 -= stepx;
                    if (d > 0) 
					{
                        Bod( x0, y0 += stepy);			    // Pattern:
                        Bod( x0 += stepx, y0 += stepy);		//      o
                        Bod( x1, y1 -= stepy);			    //    o
                        Bod( x1 -= stepx, y1 -= stepy);		//  x
                        d += incr1;
                    } else
					{
                        if (d < c)
						{
                            Bod( x0, y0);			              // Pattern:
                            Bod( x0 += stepx, y0 += stepy);       //      o
                            Bod( x1, y1);                         //  x o
                            Bod( x1 -= stepx, y1 -= stepy);       //
                        } else 
						{Bod( x0, y0 += stepy);			    // Pattern:
                            Bod( x0 += stepx, y0);			//    o o
                            Bod( x1, y1 -= stepy);			//  x
                            Bod( x1 -= stepx, y1);			//
                        }
                        d += incr2;
                    }
                }
                if (extras > 0)
				{
                    if (d > 0)
					{
                        Bod( x0 += stepx, y0 += stepy);
                        if (extras > 1) Bod( x0 += stepx, y0 += stepy);
                        if (extras > 2) Bod( x1 -= stepx, y1 -= stepy);
                    } else
                    if (d < c) 
					{Bod( x0 += stepx, y0 += stepy);
                        if (extras > 2) Bod( x1 -= stepx, y1);
                    } else 
					{
                        Bod( x0 += stepx, y0 += stepy);
                        if (extras > 1) Bod( x0 += stepx, y0);
                        if (extras > 2)
						{
                            if (d > c)
                                Bod( x1 -= stepx, y1 -= stepy);
                            else
                                Bod( x1 -= stepx, y1);
                        }
                    }
                }
            }
        } else 
        {
            int length = (dy - 1) >> 2;
            int extras = (dy - 1) & 3;
            int incr2 = (dx << 2) - (dy << 1);
            if (incr2 < 0) {
                int c = dx << 1;
                int incr1 = c << 1;
                int d =  incr1 - dy;
                for (int i = 0; i < length; i++)
				{
                    y0 += stepy;
                    y1 -= stepy;
                    if (d < 0)
					{
                        Bod( x0, y0);
                        Bod( x0, y0 += stepy);
                        Bod( x1, y1);
                        Bod( x1, y1 -= stepy);
                        d += incr1;
                    } else
					{
                        if (d < c)
						{
                            Bod( x0, y0);
                            Bod( x0 += stepx, y0 += stepy);
                            Bod( x1, y1);
                            Bod( x1 -= stepx, y1 -= stepy);
                        } else 
						{
                            Bod( x0 += stepx, y0);
                            Bod( x0, y0 += stepy);
                            Bod( x1 -= stepx, y1);
                            Bod( x1, y1 -= stepy);
                        }
                        d += incr2;
                    }
                }
                if (extras > 0)
				{
                    if (d < 0) 
					{
                        Bod( x0, y0 += stepy);
                        if (extras > 1) Bod( x0, y0 += stepy);
                        if (extras > 2) Bod( x1, y1 -= stepy);
                    } else
                    if (d < c) 
					{
                        Bod( stepx, y0 += stepy);
                        if (extras > 1) Bod( x0 += stepx, y0 += stepy);
                        if (extras > 2) Bod( x1, y1 -= stepy);
                    } else
					{
                        Bod( x0 += stepx, y0 += stepy);
                        if (extras > 1) Bod( x0, y0 += stepy);
                        if (extras > 2) Bod( x1 -= stepx, y1 -= stepy);
                    }
                }
            } else // CHYBA
			{
                int c = (dx - dy) << 1;
                int incr1 = c << 1;
                int d =  incr1 + dy;
                for (int i = 0; i < length; i++)
				{
                    y0 += stepy;
                    y1 -= stepy;
                    if (d > 0)
					{
                        Bod( x0 += stepx, y0);
                        Bod( x0 += stepx, y0 += stepy);
                        Bod( x1 -= stepy, y1);
                        Bod( x1 -= stepx, y1 -= stepy);
                        d += incr1;
                    } else 
					{
                        if (d < c) 
						{
                            Bod( x0, y0);
                            Bod( x0 += stepx, y0 += stepy);
                            Bod( x1, y1);
                            Bod( x1 -= stepx, y1 -= stepy);
                        } else 
						{
                            Bod( x0 += stepx, y0);
                            Bod( x0, y0 += stepy);
                            Bod( x1 -= stepx, y1);
                            Bod( x1, y1 -= stepy);
                        }
                        d += incr2;
                    }
                }
                if (extras > 0) 
				{
                    if (d > 0) 
					{
                        Bod( x0 += stepx, y0 += stepy);
                        if (extras > 1) Bod( x0 += stepx, y0 += stepy);
                        if (extras > 2) Bod( x1 -= stepx, y1 -= stepy);
                    } else
                    if (d < c) 
					{
                        Bod( x0, y0 += stepy);
                        if (extras > 1) Bod( x0 += stepx, y0 += stepy);
                        if (extras > 2) Bod( x1, y1 -= stepy);
                    } else 
					{
                        Bod( x0 += stepx, y0 += stepy);
                        if (extras > 1) Bod( x0, y0 += stepy);
                        if (extras > 2) 
						{
                            if (d > c)
                                Bod( x1 -= stepx, y1 -= stepy);
                            else
                                Bod( x1, y1 -= stepy);
                        }
                    }
                }
            }
        }

	Casovac->Pause();
	return Casovac->GetTime();
}
