/*-----------------------------------------------------------------------------
 
	>> INSTITUT INFORMACNICH TECHNOLOGII
	>> FAKULTA TECHNOLOGICKA
	>> UNIVERZITA TOMASE BATI VE ZLINE
	

  	Soubor	:	CRaster.h
	Popis	:	Podpurna knihovna pro rasterizovani
	Autor	:	Jan Stalmach, stalmachjan@volny.cz 
	Obor	:	Informacni technologie, 2003-2004
   

	Nastavenim promene RENDERER na pozadovanou hodnotu se ridi
	volba pouziteho renderovaciho rozhrani

	Mozne volby grafickych rozhrani:

		OPENGL			0
		WIN32API_GDI	1

 ------------------------------------------------------------------------------*/



#ifndef _RASTER_H
#define _RASTER_H		

// pi na 9 desetinnych mist
#define PI 3.141592654

// Definice podporovanych grafickych rozhrani
#define OPENGL			0
#define WIN32API_GDI	1

// Promena RENDERER nastavuje renderovaci rozhrani 
#ifndef RENDERER
#define RENDERER OPENGL     
#endif

// Definovani zakladni bunky popisujici pixel v digitalnim trojrozmernem prostoru
// Definivani zakladnich operaci s pixelem pomoci pretizenych operatoru
// BOD = (x,y,z)
struct bod
{

public:
	
	float x;
	float y;
	float z;

	// implicitni konstruktor
	bod() {}

	// volitelny konstruktor
	bod(float X, float Y, float Z) 
	{ 
		x = X; y = Y; z = Z;
	}

	// pretizeny operator + 
	bod operator+(bod bod2)
	{
		return bod(bod2.x + x, bod2.y + y, bod2.z + z);
	}

	// pretizeny operator - 
	bod operator-(bod bod2)
	{
		return bod(bod2.x - x, bod2.y - y, bod2.z - z);
	}

	// pretizeny operator / 
	bod operator/(float cislo)
	{
		return bod(x / cislo, y / cislo, z / cislo);
	}

	// pretizeny operator * (nasobeni skalarem)
	bod operator*(float cislo)
	{
		return bod(x*cislo, y*cislo, z*cislo);
	}
};


/*-------------------------------------------------------------------------------------------------------
	OPENGL
---------------------------------------------------------------------------------------------------------*/		
#if(RENDERER==OPENGL)

#include "glut.h"

// Cara pomoci knihovny OpenGL
#define LineOpenGL(x1,y1,x2,y2) glBegin(GL_LINES);glVertex2i((x1),(y1));glVertex2i((x2),(y2));glEnd(); 

// 3D cara pomoci knihovny OpenGL
#define LineOpenGL3D(x1,y1,z1,x2,y2,z2) glBegin(GL_LINES);glVertex3f((x1),(y1),(z1));glVertex3f((x2),(y2),(z2));glEnd(); 


/*--------------------------------------------------------
	Trida Raster zajistujici vykreslovani pomoci OpenGL
---------------------------------------------------------*/
class CRaster {

protected:

int barva;

public:

	inline void Barva(float r, float g, float b) { glColor3f(r,g,b); }
	inline void Bod(float x, float y) { glBegin(GL_POINTS); glVertex3f(x,y,0); glEnd(); }	
	inline void Bod3D(float x, float y, float z) { glBegin(GL_POINTS); glVertex3f(x,y,z); glEnd(); }	
};

#endif 


/*----------------------------------------------------------------------------------------------------
	WIN32 API GDI
------------------------------------------------------------------------------------------------------*/
#if(RENDERER==WIN32API_GDI)

#include <windows.h>

/*---------------------------------------------------
	Trida Raster zajistujici vykreslovani pomoci GDI
-----------------------------------------------------*/
class CRaster {

public:

	HDC hdc;

	DWORD barva;

    CRaster(){}
	inline void Barva(DWORD r, DWORD g, DWORD b) { }
	inline void Bod(int x, int y) { SetPixel(hdc,x,y,barva); }	
	inline void Bod3D(int x, int y, int z) {}		

};

#endif

#endif /* _RASTER_H */