/*-----------------------------------------------------------------------------
 
    >> INSTITUT INFORMACNICH TECHNOLOGII
	>> FAKULTA TECHNOLOGICKA
	>> UNIVERZITA TOMASE BATI VE ZLINE
					        
   	Soubor	:	CSurface.cpp
	Popis	:	Knihovna pro kresleni polygonialnich ploch
	Autor	:	Jan Stalmach, stalmachjan@volny.cz 
	Obor	:	Informacni technologie, 2004
  

    Implementovany nasledujci algoritmy:

    Bezierova bikubicka plocha
	Bezierova racionalni bikubicka plocha
	Coonsova bikubicka plocha
	Fergusonova bikubicka plocha

 ------------------------------------------------------------------------------*/



#include <math.h>
#include "CSurface.h"


/*---------------------------------------------------------------------
 konstruktor tridy CSurface
-----------------------------------------------------------------------*/
CSurface::CSurface(float r=1.0, float g=1.0, float b=1.0) 
{ 
	Casovac = new Stopky;
	Cara	= new CLine(r,g,b);
	red   = r;
	green = g;
	blue  = b;
}


/*---------------------------------------------------------------------
 nastavi vykreslovaci barvu
-----------------------------------------------------------------------*/
void CSurface::Set_RGB(float r, float g, float b)
{
	red   = r;
	green = g;
	blue  = b;
}


/*---------------------------------------------------------------------
  Bazove funkce pro Bezieruv bikubicky plat
-----------------------------------------------------------------------*/
float CSurface::Bazova_fce_Bezier(int i, float t)
{
	float pom1, pom2, pom3;

	// Pomocne promene pro zprehledneni kodu
	pom1 = 1 - t;
    pom2 = pom1 * pom1 * pom1;
    pom3 = t * t * t;

	switch (i) 
    {
		case 0: return pom2;
		case 1: return 3*t*pom1*pom1;
		case 2: return 3*t*t*pom1;
		case 3: return pom3;
	}

	return 0;
}


/*---------------------------------------------------------------------
 Vypocet bodu Bezierovy bikubiky
-----------------------------------------------------------------------*/
bod CSurface::Souradnice_Bezierova_Bikubika(bod *b, float u, float v)
{
    bod point = bod(0,0,0);

	for (int i=0; i<4; i++)
	for (int j=0; j<4; j++)
		point = point+b[i*4+j]*Bazova_fce_Bezier(i,u)*Bazova_fce_Bezier(j,v);

	// vrat novou souradnici
    return point;				
}


/*---------------------------------------------------------------------
 Bezierova bikubicka plocha
-----------------------------------------------------------------------*/
double CSurface::Bezierova_Bikubika(bod *b,int u,int v, int zpusob)
{
	Barva(red,green,blue);
	Casovac->Unpause();

	// Pole pro souradnice plochy
	bod *q = new bod[(u*v)];
	
	// Napln plochu nulama
	for(int i2 = 0; i2<(u*v); i2++) q[i2] = q[i2]*0;

	int i = 0;

	float hr = 1.0f / (u-1);
	float max_r = (1 + (1.0f / u));

	float hs = 1.0f / (v-1);
	float max_s = (1 + (1.0f / v));

	// Generovani bodu urcujicich plochu
	for(float r=0; r <= max_r; r += hr)
	{
		for(float s=0; s <= max_s; s += hs)
		{
			q[i] = Souradnice_Bezierova_Bikubika(b,r,s);
			i++;
		}
	}

	// Vykresleni plochy
	Ridici_Polygon(q,u,v,zpusob);

	Casovac->Pause();
	return Casovac->GetTime();
}


/*----------------------------------------------------------------------------------------
 Vypocet vahovych bodu Racionalni Bezierovy bikubiky
------------------------------------------------------------------------------------------*/
float CSurface::Racionalni_Bezier_Vaha(int i, int j, float u, float v, float *m)
{
	float value = 0;

	for (int a=0; a<4; a++)
	for (int b=0; b<4; b++)
		value += Bazova_fce_Bezier(a,u)*Bazova_fce_Bezier(b,v)*m[a*4+b];
	
	return (Bazova_fce_Bezier(i,u)*Bazova_fce_Bezier(j,v)*m[i*4+j])/value;
}


/*----------------------------------------------------------------------------------------
 Vypocet bodu Racionalni Bezierovy bikubiky
------------------------------------------------------------------------------------------*/
bod CSurface::Souradnice_Racionalni_Bezierova_Bikubika(bod *b, float u, float v, float *m)
{
    bod point = bod(0,0,0);

	for (int i=0; i<4; i++)
	for (int j=0; j<4; j++)
		point = point+b[i*4+j]*Racionalni_Bezier_Vaha(i,j,u,v,m);

	// vrat novou souradnici
    return point;				
}


/*---------------------------------------------------------------------------------------
 Racionalni Bezierova bikubicka plocha
-----------------------------------------------------------------------------------------*/
double CSurface::Bezierova_Racionalni_Bikubika(bod *b, float *m, int u,int v, int zpusob)
{
	Barva(red,green,blue);
	Casovac->Unpause();

	// Pole pro souradnice plochy
	bod *q = new bod[(u*v)];
	
	// Napln plochu nulama
	for(int i2 = 0; i2<(u*v); i2++) q[i2] = q[i2]*0;

	int i = 0;

	float hr = 1.0f / (u-1);
	float max_r = (1 + (1.0f / u));

	float hs = 1.0f / (v-1);
	float max_s = (1 + (1.0f / v));

	// Generovani bodu urcujicich plochu
	for(float r=0; r <= max_r; r += hr)
	{
		for(float s=0; s <= max_s; s += hs)
		{
			q[i] = Souradnice_Racionalni_Bezierova_Bikubika(b,r,s,m);
			i++;
		}
	}

	// Vykresleni plochy
	Ridici_Polygon(q,u,v,zpusob);

	Casovac->Pause();
	return Casovac->GetTime();
}


/*---------------------------------------------------------------------
  Bazove funkce pro Coonsuv bikubicky plat
-----------------------------------------------------------------------*/
float CSurface::Bazova_fce_Coons(int i, float t)
{
	float pom1, pom2, pom3;

	// Pomocne promene pro zprehledneni kodu
	pom1 = 1 - t;
	pom2 = t * t;
	pom3 = pom1 * pom1 * pom1;

	switch (i) 
    {
		case 0: return pom3 / 6;
		case 1: return (3*pom2*t - 6*pom2 + 4) / 6;
		case 2: return (-3*pom2*t + 3*pom2 +3*t +1) / 6;
		case 3: return pom2*t / 6;
	}

	return 0;
}


/*---------------------------------------------------------------------
 Vypocet bodu Coonsovy bikubiky
-----------------------------------------------------------------------*/
bod CSurface::Souradnice_Coonsova_Bikubika(bod *b, float u, float v)
{
    bod point = bod(0,0,0);

	for (int i=0; i<4; i++)
	for (int j=0; j<4; j++)
		point = point+b[i*4+j]*Bazova_fce_Coons(i,u)*Bazova_fce_Coons(j,v);

	// vrat novou souradnici
    return point;				
}


/*---------------------------------------------------------------------
 Coonsova bikubicka plocha
-----------------------------------------------------------------------*/
double CSurface::Coonsova_Bikubika(bod *b,int u,int v, int zpusob)
{
	Barva(red,green,blue);
	Casovac->Unpause();

	// Pole pro souradnice plochy
	bod *q = new bod[(u*v)];
	
	// Napln plochu nulama
	for(int i2 = 0; i2<(u*v); i2++) q[i2] = q[i2]*0;

	int i = 0;

	float hr = 1.0f / (u-1);
	float max_r = (1 + (1.0f / u));

	float hs = 1.0f / (v-1);
	float max_s = (1 + (1.0f / v));

	// Generovani bodu urcujicich plochu
	for(float r=0; r <= max_r; r += hr)
	{
		for(float s=0; s <= max_s; s += hs)
		{
			q[i] = Souradnice_Coonsova_Bikubika(b,r,s);
			i++;
		}
	}

	// Vykresleni plochy
	Ridici_Polygon(q,u,v,zpusob);

	Casovac->Pause();
	return Casovac->GetTime();
}


/*---------------------------------------------------------------------
  Bazove funkce pro Fergusonuv bikubicky plat
-----------------------------------------------------------------------*/
float CSurface::Bazova_fce_Ferguson(int i, float t)
{
	float pom1, pom2;

	// pomocne promene pro zprehledneni kodu
	pom1 = t * t;
	pom2 = t * t * t;

	// kubicke Hermitovske polynomy
	switch (i) 
    {
		case 0: return t*t*t-2*t*t+t;//2*pom2 - 3*pom1 + 1;
		case 1: return 2*t*t*t-3*t*t+1;//-2*pom2 + 3*pom1;
		case 2: return -2*t*t*t+3*t*t;//pom2 - 2*pom1 + t;
		case 3: return t*t*t-t*t;
	}

	return 0;
}


/*---------------------------------------------------------------------
 Vypocet bodu Fergusonovy bikubiky
-----------------------------------------------------------------------*/
bod CSurface::Souradnice_Fergusonova_Bikubika(bod *b, float u, float v)
{
    bod point = bod(0,0,0);

	for (int i=0; i<4; i++)
	for (int j=0; j<4; j++)
		point = point+b[i*4+j]*Bazova_fce_Ferguson(i,u)*Bazova_fce_Ferguson(j,v);

	// vrat novou souradnici
    return point;				
}


/*---------------------------------------------------------------------
 Fergusonova bikubicka plocha
-----------------------------------------------------------------------*/
double CSurface::Fergusonova_Bikubika(bod *b,int u,int v, int zpusob)
{
	Barva(red,green,blue);
	Casovac->Unpause();

	// Pole pro souradnice plochy
	bod *q = new bod[(u*v)];
	
	// Napln plochu nulama
	for(int i2 = 0; i2<(u*v); i2++) q[i2] = q[i2]*0;

	int i = 0;

	float hr = 1.0f / (u-1);
	float max_r = (1 + (1.0f / u));

	float hs = 1.0f / (v-1);
	float max_s = (1 + (1.0f / v));

	// Generovani bodu urcujicich plochu
	for(float r=0; r <= max_r; r += hr)
	{
		for(float s=0; s <= max_s; s += hs)
		{
			q[i] = Souradnice_Fergusonova_Bikubika(b,r,s);
			i++;
		}
	}

	// Vykresleni plochy
	Ridici_Polygon(q,u,v,zpusob);

	Casovac->Pause();
	return Casovac->GetTime();
}



/*-----------------------------------------------------------------------
 Ridici polygon plochy
-----------------------------------------------------------------------*/
void CSurface::Ridici_Polygon(bod *b,int n, int m, int zpusob)
{
	if (zpusob==SIT)
	{
		// napred spojim radky
		for(int c = 0; c<n*n; c+=n)
		{
			for(int radek = 0; radek<n-1; radek++)
			{
				// Vykresleni 3D cary pomoci OpenGL
				LineOpenGL3D(b[c+radek].x,b[c+radek].y,b[c+radek].z,b[c+radek+1].x,b[c+radek+1].y,b[c+radek+1].z);		
			}
		}
	
		// a pak sloupce
		for(int c2 = 0; c2<m; c2++)
		{
			for(int radek2 = 0; radek2<m*n-m; radek2+=n)
			{
				// Vykresleni 3D cary pomoci OpenGL
				LineOpenGL3D(b[c2+radek2].x,b[c2+radek2].y,b[c2+radek2].z,b[c2+radek2+n].x,b[c2+radek2+n].y,b[c2+radek2+n].z);		
			}
		}	

	}

	if (zpusob==BODY)
	{
		for(int i = 0; i<(n*m); i++) Bod3D(b[i].x,b[i].y,b[i].z);	
	}
}
