/*-----------------------------------------------------------------------------
 
    >> INSTITUT INFORMACNICH TECHNOLOGII
	>> FAKULTA TECHNOLOGICKA
	>> UNIVERZITA TOMASE BATI VE ZLINE
					        
   	Soubor	:	CSurface.h
	Popis	:	Knihovna pro kresleni polynomialnich parametrickych ploch
	Autor	:	Jan Stalmach, stalmachjan@volny.cz 
	Obor	:	Informacni technologie, 2004
  

    Implementovany nasledujci algoritmy:

 	Bezierova bikubicka plocha
	Bezierova racionalni bikubicka plocha
	Coonsova bikubicka plocha
	Fergusonova bikubicka plocha


	POZNAMKA:

	Tento objekt pro vykreslovani prostorovych ploch lze spoustet pouze
	pod rozhranim OpenGL. Standartni WinAPI neposkytuje funkce pro kresbu usecky
	v prostoru. 
	
	Pro prostorove vykresleni usecky ve Windows (nepouzivate-li OpenGL) 
	lze s vyhodou pouzit rozhrani DirectX. Staci pouze modifikovat 
	soubor CRaster.h a zmenit kreslici funkci v CSurface:Ridici_Polygon.

 ------------------------------------------------------------------------------*/

#ifndef _CSURFACE_H		
#define _CSURFACE_H		

#include "CRaster.h"
#include "stopky.h"
#include "CLine.h"
#include <iostream>

// Zpusob vykreslovani polygonialni site
#define BODY 1
#define SIT  2


/*----------------------------------------------
	Trida CSurface
------------------------------------------------*/
class CSurface: public CRaster {

protected:

	Stopky *Casovac;

	float red,green,blue;

	float Bazova_fce_Bezier(int i, float t);
	float Bazova_fce_Coons(int i, float t);
	float Racionalni_Bezier_Vaha(int i, int j, float u, float v, float *m);
	float Bazova_fce_Ferguson(int i, float t);

	bod   Souradnice_Bezierova_Bikubika(bod *b, float u, float v);
	bod   Souradnice_Racionalni_Bezierova_Bikubika(bod *b, float u, float v,float *m);
	bod   Souradnice_Coonsova_Bikubika(bod *b, float u, float v);
	bod   Souradnice_Fergusonova_Bikubika(bod *b, float u, float v);


public:

	CLine   *Cara;

	CSurface(float r, float g, float b);
	~CSurface() {}

	double Bezierova_Bikubika(bod *b,int u,int v, int zpusob);
	double Bezierova_Racionalni_Bikubika(bod *b,float *m, int u,int v, int zpusob);
	double Coonsova_Bikubika(bod *b,int u,int v, int zpusob);
	double Fergusonova_Bikubika(bod *b,int u,int v, int zpusob);

	void Ridici_Polygon(bod *b,int n, int m, int zpusob);
	void Set_RGB(float r, float g, float b);
};

#endif