/*-----------------------------------------------------------------------------
 
  UNIVERZITA TOMASE BATI VE ZLINE
  FAKULTA TECHNOLOGICKA
	
  Soubor : stopky.cpp
  Nazev  : Knihovna pro mereni casu pod Win32 [presnost tisiciny ms] 
           a pod multiplatformnim SDL [presnost ms]
  Autor  : Jan Stalmach, 2003-2004, stalmachjan@volny.cz
  Obor   : Informacni technologie

 ------------------------------------------------------------------------------*/

#include "stopky.h"

#ifdef WIN32
#include <windows.h>

#elif

#include "SDL.h"
#include "SDL_timer.h"
	
#endif 	
	
/*----------------------------------
 Konstruktor casovace
------------------------------------*/
Stopky::Stopky()
{
	#ifdef WIN32
	if ( QueryPerformanceFrequency(( LARGE_INTEGER* )&i64Freq ))		// Zkus nacist frekvenci casovace
	{
		QueryPerformanceCounter(( LARGE_INTEGER* )&i64TimerStart );		// Ziskej aktualni cas a uloz do promenne m_timer_start
		fResolutionTime = ( float )( 1.0/( double )i64Freq ) * 1000.0f;	// Vypocti casovy rozlozeni podle frekvence casovace
	}

	iPauseCount = 0;													// Nastav velikost casoveho useku zastaveni casovace na 0 
	dPauseTime = 0;														// Nastav cas, kdy doslo k zastaveni casovace na 0

	uiMinCountTick = 100000000;											// Min pocet taktu CPU casovace
	#endif

}

/*----------------------------------
 Resetuj cas na 0
------------------------------------*/
void Stopky::Reset()
{
	#ifdef WIN32
	QueryPerformanceCounter(( LARGE_INTEGER* )&i64TimerStart );			// Ziskej aktualni cas a uloz do promenne m_timer_start
	dPauseTime = GetTime();												// Do m_pause_time uloz hodnotu aktualniho casu
	#endif
}


/*----------------------------------
 Ziskej cas v milisekundch
------------------------------------*/
double Stopky::GetTime() const
{
	#ifdef WIN32
	if ( iPauseCount > 0 )												// Pokud je velikost casoveho useku zastaveni casovace vetsi ne 0, tak											
		return dPauseTime;												// Vrat funkci hodnotu casu, kdy doslo k zastaveni casovace

	__int64 i64_time_elapsed;											// Promenne uplynuleho casu

	QueryPerformanceCounter(( LARGE_INTEGER* )&i64_time_elapsed );		// Ziskej aktualni cas a uloz do promenne time_elapsed
	i64_time_elapsed -= i64TimerStart;									// Odecti uplynuly cas od casu pri startu casovace
	return i64_time_elapsed * fResolutionTime;							// Vrat funkci vysledny cas v milisekundach

	#elif
	return sTime-iPauseCount;
	#endif 	

  }


/*----------------------------------
 Zastav casovac
------------------------------------*/
void Stopky::Pause()
{
	#ifdef WIN32
	if ( iPauseCount == 0 )			// Pokud je velikost casoveho useku zastaveni casovace rovna nule, tak									
		dPauseTime = GetTime();		// Vrat funkci hodnotu casu, kdy doslo k zastaveni casovace

	iPauseCount++;					// Pricitej velikost casoveho useku zastaveni casovace

	#elif
	sTime = SDL_GetTicks();
	#endif
}


/*----------------------------------
 Znovu rozjed zastaveni casovace
------------------------------------*/
void Stopky::Unpause()
{
	#ifdef WIN32
	if( iPauseCount > 0 )			// Jestlize je casovy usek zastaveneho casovace vetsi nez 0, tak
		iPauseCount--;				// Odecitej hodnotu		

	if ( iPauseCount == 0 )			// Jestlize je casovy usek zastaveneho casovace roven 0, tak
	{
		__int64 i64_time;															// Promenna casu

		QueryPerformanceCounter(( LARGE_INTEGER* )&i64_time );						// Ziskej aktualni cas a uloz do promenne time
		i64TimerStart = i64_time - ( __int64 )( iPauseCount / fResolutionTime );	// Rozjed casovac
	}

	#elif
	sTime = 0;
	iPauseCount = SDL_GetTicks();
	#endif
}




